package com.chelvc.framework.database.support;

import java.util.List;
import java.util.function.Function;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import lombok.NonNull;

/**
 * Lambda查询包装器增强实现
 *
 * @author Woody
 * @date 2024/3/18
 */
public class EnhanceLambdaQueryWrapper<T> extends LambdaQueryChainWrapper<T> {
    public EnhanceLambdaQueryWrapper(BaseMapper<T> mapper) {
        super(mapper);
    }

    /**
     * 分页查询
     *
     * @param paging 查询分页信息
     * @return 数据分页信息
     */
    public Pagination<T> paging(@NonNull Paging paging) {
        return DatabaseContextHolder.paging(paging, this::page);
    }

    /**
     * 分页查询
     *
     * @param paging  查询分页信息
     * @param adapter 数据转换适配器
     * @param <R>     目标数据类型
     * @return 数据分页信息
     */
    public <R> Pagination<R> paging(@NonNull Paging paging, @NonNull Function<List<T>, List<R>> adapter) {
        return DatabaseContextHolder.paging(paging, this::page, adapter);
    }
}
